;;;
;;; Makefile for jscheme.
;;; 
(load "build/compile.scm")
(load "elf/classpath.scm")

(define srcDir (File. "src"))
(define classDir srcDir)
(define (src name) (File. srcDir name))
(define bin (find-javac-bin))
(define directories
  ;; Compile java in these directories recursively.
  '("build"				; references sun.tools.javac.Main,
    "dclass"
    "elf"
    "interact"
    "jlib"
    "jscheme"
    "jschemeweb"			; references Servlet	
    "jsint"))

(define compile-cp
  ;; Grow the compiling classpath to include tools like javac and jar.
  ;; and any lib/*.jar or lib/*.zip files.
  (path classDir
	(File. (.getParent bin) "lib/tools.jar")
	(files (File. "lib") jar-file?)))

(define (javac-message dir files)
  (let ((L (length files)))
    (display
     {Javacing [L] file[(if (= L 1) "" "s")][(if dir { in [dir]} "")]\n})))

(define (javac . files)
  (let ((files (needs-javac (map src files))))
    (if (> (length files) 0)
	(begin 
	  (javac-message #f files)
	  (run (File. bin "javac") "-g" "-classpath" compile-cp
	       "-d" classDir files))
	#t)))

(define (javac-all dir)
  ;; Compile all files below directory.
  (let ((files (needs-javac (files (src dir) java-file?))))
    (if (> (length files) 0)
	(begin
	  (javac-message dir files)
	  (run (File. bin "javac") "-g" "-classpath" compile-cp
	       "-d" classDir files))
	#t)))

(define java  (list (File. bin "java")  "-classpath" classpath))

(define (compile class package)
  ;; .scm -> .java -> .class
  (display {Compiling [class] to package [package]\n})
  (compile-file (.toString (src (string-append class ".scm"))) package)
  (javac (string-append class ".java")))

(define (bload f)
  (load f)
  #t)

(define (-C dir files)
  (map (lambda (f) (list "-C" dir (relativize dir f))) files))

(define (relativize dir file)
  (File. (.substring (.toString file) (.length (.toString dir)))))
(assert (.equals
	 (relativize (File. "src") (File. "src/build/jscheme-bootstrap.scm"))
	 (File. "/build/jscheme-bootstrap.scm")))

(define (needs-javac files)
  (filter
   (lambda (file)
     (needs-update? (list (java->class classDir srcDir file)) (list file)))
	  files))

(define (java->class classDir srcDir file)
  (let ((f (.toString (relativize srcDir file))))
    (File. (string-append classDir
			  (.substring f 0 (- (.length f) (.length ".java")))
			  ".class"))))

(define (make-compiler)
  ;; Compile the .scm -> .java compiler.
  (display {Compiling the compiler.})
  (and
   (bload "jsint/compiler/Reflect.scm")
   (bload "jsint/compiler/CompileReflection.scm")
   ;; compile utility procedures
   (javac "jsint/Function.java" "jsint/LCO.java")
   (bload "jsint/compiler/Compiler.scm") ; Needs jsint.Function.
   (compile "jsint/compiler/Reflect" "jsint.compiler")
   (compile "jsint/compiler/CompileReflection" "jsint.compiler")

   ;; compile the compiler
   (compile "jsint/compiler/Compiler" "jsint.compiler")
   ;; compile the toplevel interface to compiler
   (compile "jsint/Compile" "jsint")
   ;; Load the compiler.
   ;; KRA 24APR02: This doesn't work: (jsint.Compile.load) 
   ))

(define (make-jlib)
  ;; compile JLIB and Newtworking.
  (and
   (javac "jsint/SchemeApplet.java" "jlib/SchemeCanvas.java")
   (compile "jlib/JLIB" "jlib")
   (compile "jlib/Swing" "jlib")
   (compile "jlib/Networking" "jlib")
   (compile "jlib/SNLP" "jlib")
   ))

(define (test-jar)
  ;; Run tests on lib/jscheme.jar
  (run `(,java -classpath lib/jscheme.jar jscheme.REPL
	       jscheme/SchemeTests.scm "(exit)")))

(define (make-jar)
  ;; Make lib/jscheme.jar
  (and 
   (bload "jsint/listener.scm")
   (bload "jsint/primitives.scm")
   (javac-all "jsint")
   (make-compiler)
   (make-jlib)
   (for-each javac-all directories)
   (run (File. bin "jar") "cvfm" "lib/jscheme.jar" (src "jsint/manifest")
	(-C classDir (files classDir class-file?))
	(-C srcDir (files srcDir scheme-file?)))
   (test-jar)
   (display {\nlib/jscheme.jar built.\n})
   (run (File. bin "jar") "cvf" "lib/applet.jar"
	(-C classDir
	    (flatten
	     (map (lambda (d) (files (File. classDir d) class-file?))
		  '("jlib" "jsint" "jscheme")))))
   (display {lib/applet.jar built.\n})
   (exit)))

